package hamiltonianGraph;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import javax.swing.*;

public class GraphPanel extends JComponent {
    private static final int WIDE = 1250;
    private static final int HIGH = 600;
    private static final int RADIUS = 20;
    private ControlPanel control = new ControlPanel();
    private Generator generator = new Generator();
    private int radius = RADIUS;
    private List<Node> nodes = new ArrayList<>();       //vizuální uzly
    private List<Node> selected = new ArrayList<>();    //vybrané uzly
    private List<Edge> edges = new ArrayList<>();       //vizuální hrany
    private Point mousePt = new Point(WIDE / 2, HIGH / 2);
    private Rectangle mouseRectangle = new Rectangle();     //pro označování uzlů
    private boolean selecting = false;
    private Controller controller = new Controller();
    private Example example = new Example(controller);
    private Point lastMousePt;
    private int shiftPosition = (int)(radius * 1.5); // aby se nove uzly neprekryvaly
    private int vertexCountForGenerate = 6;
    private String result;
    private List<Edge> resultEdges = new ArrayList<>();     //výsledné hrany ham. kružnice
    public int switchLanguage = 0;

    public GraphPanel() {
        this.setOpaque(true);
        this.addMouseListener(new MouseHandler());
        this.addMouseMotionListener(new MouseMotionHandler());
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(WIDE, HIGH);
    }

    //vykreslování
    @Override
    public void paintComponent(Graphics g) {
        g.setColor(new Color(0x00f0f0f0));
        g.fillRect(0, 0, getWidth(), getHeight());
        for (Edge e : edges) {
            e.draw(g);
        }

        //pro obarvení hran výsledku
        if (resultEdges.size() != 0) {
            for (Edge en : resultEdges) {
                en.draw(g, Color.green);
            }
        }

        for (Node n : nodes) {
            n.draw(g);
        }
        if (selecting) {
            g.setColor(Color.darkGray);
            g.drawRect(mouseRectangle.x, mouseRectangle.y,
                    mouseRectangle.width, mouseRectangle.height);
        }
    }

    private class MouseHandler extends MouseAdapter {

        @Override
        public void mouseReleased(MouseEvent e) {
            selecting = false;
            mouseRectangle.setBounds(0, 0, 0, 0);
            if (e.isPopupTrigger()) {
                showPopup(e);
            }
            e.getComponent().repaint();
        }

        @Override
        public void mousePressed(MouseEvent e) {
            mousePt = e.getPoint();

            if (e.isPopupTrigger()) {
                Node.selectOne(nodes, mousePt);
                showPopup(e);
            } else if (Node.selectOne(nodes, mousePt)) {
                selecting = false;
            } else {
                Node.selectNone(nodes);
                selecting = true;
            }
            e.getComponent().repaint();
        }

        private void showPopup(MouseEvent e) {
            control.popup.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    private class MouseMotionHandler extends MouseMotionAdapter {

        Point delta = new Point();

        @Override
        public void mouseDragged(MouseEvent e) {
            if (selecting) {
                mouseRectangle.setBounds(
                        Math.min(mousePt.x, e.getX()),
                        Math.min(mousePt.y, e.getY()),
                        Math.abs(mousePt.x - e.getX()),
                        Math.abs(mousePt.y - e.getY()));
                Node.selectRectangle(nodes, mouseRectangle);
            } else {
                delta.setLocation(
                        e.getX() - mousePt.x,
                        e.getY() - mousePt.y);
                Node.updatePosition(nodes, delta);
                mousePt = e.getPoint();
            }
            e.getComponent().repaint();
            lastMousePt = mousePt;
            shiftPosition = (int)(radius * 1.5);
        }
    }

    public JToolBar getControlPanel() {
        return control;
    }

    public class ControlPanel extends JToolBar {
        //ovládací prvky
        private Action newNode = new NewNodeAction("New");
        public Action clearAll = new ClearAction("Clear");
        private Action generate = new GenerateAction("Generate");
        private Action delete = new DeleteAction("Delete");
        private Action save = new SaveAction("Save");
        private Action solve = new SolveAction("Solve");
        private Action connect = new ConnectAction("Connect");
        private Action prepared = new PreparedAction("Examples");
        private Action example1 = new Example1Action("Example 1");
        private Action example2 = new Example2Action("Example 2");
        private Action example3 = new Example3Action("Example 3");
        private Action example4 = new Example4Action("Example 4");
        private Action example5 = new Example5Action("Example 5");
        private Action example6 = new Example6Action("Example 6");
        private Action example7 = new Example7Action("Example 7");
        private Action example8 = new Example8Action("Example 8");
        private Action help = new HelpAction("Help");
        private Action language = new LangAction("CZ/EN");
        private JPopupMenu popup = new JPopupMenu();
        private JLabel resultL = new JLabel("Answer");
        private JMenuBar menuBar = new JMenuBar();
        private JMenu menu = new JMenu(prepared);
        private JLabel sizeLabel = new JLabel("Size:");
        private JLabel countLabel = new JLabel("Vertex count:");

        ControlPanel() {
            this.setLayout(new FlowLayout(FlowLayout.LEFT));
            this.setBackground(Color.lightGray);

            JSpinner jsSize = new JSpinner();
            jsSize.setModel(new SpinnerNumberModel(RADIUS, 5, 100, 5));
            jsSize.addChangeListener(e -> {
                JSpinner s = (JSpinner) e.getSource();
                radius = (Integer) s.getValue();
                Node.updateRadius(nodes, radius);
                GraphPanel.this.repaint();
            });

            JSpinner jsVertex = new JSpinner();
            jsVertex.setModel(new SpinnerNumberModel(6, 3, 10, 1));
            jsVertex.addChangeListener(e -> {
                JSpinner s = (JSpinner) e.getSource();
                controller.setVertexCount((Integer) s.getValue());
                vertexCountForGenerate = controller.vertexCount;
                GraphPanel.this.repaint();
            });
            this.add(new JButton(language));
            this.add(new JButton(help));
            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(new JButton(newNode));
            this.add(new JButton(delete));
            this.add(new JButton(connect));
            this.add(new JButton(save));
            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(new JButton(generate));
            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(new JButton(clearAll));
            this.add(new JButton(solve));
            this.add(Box.createRigidArea(new Dimension(5, 0)));

            this.add(menuBar);
            menuBar.add(menu);
            menu.add(new JMenuItem(example1));
            menu.add(new JMenuItem(example2));
            menu.add(new JMenuItem(example3));
            menu.add(new JMenuItem(example4));
            menu.add(new JMenuItem(example5));
            menu.add(new JMenuItem(example6));
            menu.add(new JMenuItem(example7));
            menu.add(new JMenuItem(example8));

            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(sizeLabel);
            this.add(jsSize);
            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(countLabel);
            this.add(jsVertex);
            this.add(Box.createRigidArea(new Dimension(5, 0)));
            this.add(resultL);
            popup.add(new JMenuItem(newNode));
            popup.add(new JMenuItem(connect));
            popup.add(new JMenuItem(delete));
        }
    }

    // nový uzel
    private class NewNodeAction extends AbstractAction {

        public NewNodeAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            // kresleni pres New, odsazuje nove body, aby nebyly na stejnem miste
            if (nodes.size() == 0) {
                createNode(mousePt.getLocation());
                shiftPosition = (int)(radius * 1.5);

            } else if (lastMousePt.x == (mousePt.getLocation().x) && lastMousePt.y == (mousePt.getLocation().y)) {
                Point p = new Point(mousePt.getLocation().x + shiftPosition, mousePt.getLocation().y + shiftPosition);
                createNode(p);

                shiftPosition = (int)(radius * 1.5) + shiftPosition;
            } else {
                createNode(mousePt.getLocation());
                shiftPosition = (int)(radius * 1.5);
            }
            lastMousePt = mousePt;
            control.save.setEnabled(true);
            control.solve.setEnabled(false);
        }
    }

    //ukázkové příklady
    private static class PreparedAction extends AbstractAction {
        public PreparedAction(String name) { super(name); }
        public void actionPerformed(ActionEvent e) {
        }
    }

    private class Example1Action extends AbstractAction {
        public Example1Action(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample1();
            btnEnableConfig(true);

        }
    }

    private class Example2Action extends AbstractAction {
        public Example2Action(String name) { super(name); }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample2();
            btnEnableConfig(true);
        }
    }

    public class Example3Action extends AbstractAction {
        public Example3Action(String name) { super(name); }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample3();
            btnEnableConfig(true);
        }
    }

    private class Example4Action extends AbstractAction {
        public Example4Action(String name) { super(name); }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample4();
            btnEnableConfig(true);
        }
    }

    private class Example5Action extends AbstractAction {
        public Example5Action(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample5();
            btnEnableConfig(true);
        }
    }

    private class Example6Action extends AbstractAction {
        public Example6Action(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample6();
            btnEnableConfig(true);
        }
    }

    private class Example7Action extends AbstractAction {
        public Example7Action(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample7();
            btnEnableConfig(true);
        }
    }

    private class Example8Action extends AbstractAction {
        public Example8Action(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            example.getExample8();
            btnEnableConfig(true);
        }
    }
    //nápověda
    private class HelpAction extends AbstractAction {
        public HelpAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            helpFrame();
        }
    }
    // změna jazyka
    private class LangAction extends AbstractAction {
        public LangAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            if (switchLanguage == 0) {
                control.help.putValue(Action.NAME, "Nápověda");
                control.newNode.putValue(Action.NAME, "Nový");
                control.delete.putValue(Action.NAME, "Smazat");
                control.save.putValue(Action.NAME, "Uložit");
                control.generate.putValue(Action.NAME, "Generovat");
                control.prepared.putValue(Action.NAME, "Příklady");
                control.connect.putValue(Action.NAME, "Spojit");
                control.clearAll.putValue(Action.NAME, "Smazat vše");
                control.example1.putValue(Action.NAME, "Příklad 1");
                control.example2.putValue(Action.NAME, "Příklad 2");
                control.example3.putValue(Action.NAME, "Příklad 3");
                control.example4.putValue(Action.NAME, "Příklad 4");
                control.example5.putValue(Action.NAME, "Příklad 5");
                control.example6.putValue(Action.NAME, "Příklad 6");
                control.example7.putValue(Action.NAME, "Příklad 7");
                control.example8.putValue(Action.NAME, "Příklad 8");
                control.solve.putValue(Action.NAME, "Vyřešit");
                control.sizeLabel.setText("Velikost:");
                control.countLabel.setText("Počet uzlů:");
                if (control.resultL.getText().equals("Answer")) {
                    control.resultL.setText("Odpověď");
                } else if (control.resultL.getText().equals("Too few vertexes. Hamiltonian graph does not exist.")) {
                    control.resultL.setText("Příliž málo uzlů. Hamiltonovský graf nemůže existovat.");
                } else if (control.resultL.getText().equals("Hamiltonian graph exists.")) {
                    control.resultL.setText("Hamiltonovský graf existuje.");
                } else if (control.resultL.getText().equals("Hamiltonian graph does not exist.")) {
                    control.resultL.setText("Hamiltonovský graf neexistuje.");
                }
                switchLanguage = 1;
            } else if (switchLanguage == 1) {
                control.help.putValue(Action.NAME, "Help");
                control.newNode.putValue(Action.NAME, "New");
                control.delete.putValue(Action.NAME, "Delete");
                control.save.putValue(Action.NAME, "Save");
                control.generate.putValue(Action.NAME, "Generate");
                control.prepared.putValue(Action.NAME, "Examples");
                control.connect.putValue(Action.NAME, "Connect");
                control.clearAll.putValue(Action.NAME, "Clear");
                control.example1.putValue(Action.NAME, "Example 1");
                control.example2.putValue(Action.NAME, "Example 2");
                control.example3.putValue(Action.NAME, "Example 3");
                control.example4.putValue(Action.NAME, "Example 4");
                control.example5.putValue(Action.NAME, "Example 5");
                control.example6.putValue(Action.NAME, "Example 6");
                control.example7.putValue(Action.NAME, "Example 7");
                control.example8.putValue(Action.NAME, "Example 8");
                control.solve.putValue(Action.NAME, "Solve");
                control.sizeLabel.setText("Size:");
                control.countLabel.setText("Vertex count:");
                if (control.resultL.getText().equals("Odpověď")) {
                    control.resultL.setText("Answer");
                } else if (control.resultL.getText().equals("Příliž málo uzlů. Hamiltonovský graf nemůže existovat.")) {
                    control.resultL.setText("Too few vertexes. Hamiltonian graph does not exist.");
                } else if (control.resultL.getText().equals("Hamiltonovský graf existuje.")) {
                    control.resultL.setText("Hamiltonian graph exists.");
                } else if (control.resultL.getText().equals("Hamiltonovský graf neexistuje.")) {
                    control.resultL.setText("Hamiltonian graph does not exist.");
                }
                switchLanguage = 0;
            }
        }

    }
    // smazání všeho
    private class ClearAction extends AbstractAction {
        public ClearAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
        }
    }

    private void clearAll() {
        controller.vertexList.clear();
        nodes.clear();
        edges.clear();
        resultEdges.clear();
        controller.solvedVertexes.clear();
        controller.backStepFlag = -2;
        if (switchLanguage == 0) {
            control.resultL.setText(result = "Answer");
        } else {
            control.resultL.setText(result = "Odpověď");
        }
        repaint();

        control.save.setEnabled(false);
        control.delete.setEnabled(true);
        control.newNode.setEnabled(true);
        control.connect.setEnabled(true);
    }

    //náhodné generování
    private class GenerateAction extends AbstractAction {

        public GenerateAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            clearAll();
            // pomuze vygenerovat tolik uzlu, jako je uvedeno v uzivatelskem rozhrani
            if (controller.vertexCount != vertexCountForGenerate) {
                controller.setVertexCount(vertexCountForGenerate);
            }
            controller.generate();
            prepareNode(true);
            prepareEdges();
            control.solve.setEnabled(true);
            btnEnableConfig(false);
        }
    }
    //spustí prohledávání grafu
    private class SolveAction extends AbstractAction {

        public SolveAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            boolean errorFlag = false;
            if (nodes.size() == 1 || nodes.size() == 2) {
                if (switchLanguage == 0) {
                    result = ("Too few vertexes. Hamiltonian graph does not exist.");
                } else {
                    result = ("Příliž málo uzlů. Hamiltonovský graf nemůže existovat.");
                }

                //pokud neni uzel nedelej nic, pokud 1 až 2 - ham. neexistuje nehledej, jinak hledej
            } else if (nodes.size() != 0) {
                boolean exist = false;
                try {
                    exist = controller.search();
                } catch (StackOverflowError en) {
                    if (switchLanguage == 0) {
                        JOptionPane.showMessageDialog(control,
                                "Graph is too difficult, programme can not solve its.",
                                "Warning", JOptionPane.WARNING_MESSAGE);
                    } else {
                        JOptionPane.showMessageDialog(control,
                                "Příliš složitý graf, program ho nedokáže vyhodnotit.",
                                "Varování", JOptionPane.WARNING_MESSAGE);
                    }
                    errorFlag = true;
                }

                if (!errorFlag) {
                    if (exist) {
                        if (switchLanguage == 0) {
                            result = ("Hamiltonian graph exists.");
                        } else {
                            result = ("Hamiltonovský graf existuje.");
                        }

                        for (int i = 0; i < nodes.size(); i++) {
                            setColorNode(i, Color.green);
                        }
                    } else {
                        if (switchLanguage == 0) {
                            result = ("Hamiltonian graph does not exist.");
                        } else {
                            result = ("Hamiltonovský graf neexistuje.");
                        }
                    }
                }
            }

            if (!errorFlag) {
                control.resultL.setText(result);
                coloredEdges();
            }

            control.solve.setEnabled(false);
            control.save.setEnabled(false);
            controller.init();
        }
    }

    // propojení hranami
    private class ConnectAction extends AbstractAction {

        public ConnectAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            boolean duplicityFlag = false;
            Node.getSelected(nodes, selected);
            if (selected.size() > 1) {
                for (int i = 0; i < selected.size() - 1; ++i) {
                    Node n1 = selected.get(i);
                    Node n2 = selected.get(i + 1);

                    // zabrani duplicitnim hranam mezi stejnymi body
                    for (int d = 0; d < edges.size() - 1; d++) {
                        if ((n1 == edges.get(d).getN1() && n2 == edges.get(d).getN2())
                                || (n2 == edges.get(d).getN1() && n1 == edges.get(d).getN2())) {
                            duplicityFlag = true;
                        }
                    }
                    if (!duplicityFlag) {
                        edges.add(new Edge(n1, n2));
                    }
                    duplicityFlag = false;
                }
            }
            repaint();
        }
    }

    //smazání označených uzlů
    private class DeleteAction extends AbstractAction {
        public DeleteAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            control.solve.setEnabled(false);
            ListIterator<Node> iter = nodes.listIterator();
            while (iter.hasNext()) {
                Node n = iter.next();
                if (n.isSelected()) {
                    deleteEdges(n);
                    iter.remove();
                    shiftPosition -= (int)(radius * 1.5);
                    controller.vertexCount--;
                }
            }
            repaint();
        }

        private void deleteEdges(Node n) {
            edges.removeIf(e -> e.getN1() == n || e.getN2() == n);
        }
    }

    // prřipravení nakresleného grafu pro prohledávání
    private class SaveAction extends AbstractAction {
        public SaveAction(String name) {
            super(name);
        }

        public void actionPerformed(ActionEvent e) {
            btnEnableConfig(false);

            if (nodes.size() != 0) {
                controller.generateCustom(nodes, edges);
            }
        }
    }

    // vytvoření uzlu
    public void createNode(Point point) {
        Node.selectNone(nodes);
        Color color = Color.GRAY;
        Node n = new Node(point, radius, color);
        n.setSelected(true);
        nodes.add(n);
        repaint();
    }

    // vytvoření uzlu a napojení na vertex
    public void createNode(Point point, Vertex vertex) {
        Node.selectNone(nodes);
        Color color = Color.blue;
        Node n = new Node(point, radius, color, vertex);
        n.setSelected(true);
        nodes.add(n);
        repaint();
    }

    // barva uzlu
    public void setColorNode(int positionInList, Color color) {
        Node.selectNone(nodes);
        nodes.get(positionInList).setColor(color);
        repaint();
    }

    // propojení node s vertexem
    public void prepareNode(boolean IsRadomize) {
        int xv;
        int yv;
        for (int i = 0; i < controller.vertexCount; i++) {
            xv = controller.vertexList.get(i).getX();
            yv = controller.vertexList.get(i).getY();

            if (IsRadomize) {
                Point p = new Point(generator.noCoverInGenerate(WIDE, HIGH, nodes));
                xv = p.x;
                yv = p.y;
            }
            createNode(new Point(xv, yv), controller.vertexList.get(i));
        }
    }

    // vykresleni hran
    public void prepareEdges() {
        for (int i = 0; i < controller.vertexCount; i++) {
            for (int j = 0; j < controller.vertexCount; j++) {
                if (controller.path[i][j] == 1) {
                    edges.add(new Edge(nodes.get(i), nodes.get(j)));
                }
            }
        }
    }

    // obarveni hran
    public void coloredEdges() {
        for (int i = 0; i < (controller.solvedVertexes.size() - 1); i++) {

            for (Edge edge : edges) {
                if (edge.getN1().getVertex().getX() == controller.solvedVertexes.get(i).getX()
                        && edge.getN1().getVertex().getY() == controller.solvedVertexes.get(i).getY()
                        && edge.getN2().getVertex().getX() == controller.solvedVertexes.get(i + 1).getX()
                        && edge.getN2().getVertex().getY() == controller.solvedVertexes.get(i + 1).getY()) {
                    resultEdges.add(edge);
                    break;
//
                } else if (edge.getN2().getVertex().getX() == controller.solvedVertexes.get(i).getX()
                        && edge.getN2().getVertex().getY() == controller.solvedVertexes.get(i).getY()
                        && edge.getN1().getVertex().getX() == controller.solvedVertexes.get(i + 1).getX()
                        && edge.getN1().getVertex().getY() == controller.solvedVertexes.get(i + 1).getY()) {
                    resultEdges.add(edge);
                    break;
                }
            }
        }
    }

    // nastavení dostupnosti tlačítek
    private void btnEnableConfig(boolean exampleSet) {
        // donastavení věcí pro ukázkové příklady, když false je to pro generate, jinak pro example
        if (exampleSet) {
            prepareNode(false);
            prepareEdges();
        }
        control.solve.setEnabled(true);
        control.delete.setEnabled(false);
        control.newNode.setEnabled(false);
        control.connect.setEnabled(false);
    }

    // okno napovedy
    private void helpFrame() {
        EventQueue.invokeLater(() -> {
            JFrame frame = new JFrame("Help");
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

            JPanel panel = new JPanel();
            panel.setLayout(new BorderLayout());
            JTextArea textArea = new JTextArea(25, 50);
            textArea.setEditable(false);
            textArea.setFont(textArea.getFont().deriveFont(14f));
            textArea.setText(getHelpString());

            panel.add(textArea, BorderLayout.LINE_START);

            JScrollPane scroller = new JScrollPane(textArea);
            scroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
            scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

            panel.add(scroller, BorderLayout.CENTER);
            frame.getContentPane().add(BorderLayout.CENTER, panel);
            frame.pack();
            frame.setSize(960, 560);
            frame.setLocationRelativeTo(null);
            frame.setVisible(true);

        });
    }

    //text nápovědy
    private String getHelpString() {
        return ("\n  Aplikace slouží pro hledání hamiltonovských grafů za pomoci backtrackingu." +
                " Hamiltonovský graf existuje, když lze projít všemi body právě jednou \n" +
                "  a vrátit se do výchozího bodu. Grafy lze kreslit nebo generovat. Případně lze použít některý z předpřipravených příkladů. \n" +
                "\n  Ovládání:\n  Levé tlačítko myši označí jeden uzel. Více uzlů lze označit tažením myši." +
                " Stisknutí pravého tlačítka myši otevře vertikální menu" +
                "\n  s tlačítky New, Delete, Connect.\n" +
                "\n  Vlastnosti tlačítek:\n" +
                "  CZ/EN – změna jazyka: angličtina nebo čeština, Nový – vytvoří nový uzel, Smazat – smaže označené uzly," +
                " Spojit - propojí označené uzly hranami,\n" +
                "  Uložit – uloží nakreslený graf, aby mohl být prohledán, Generovat – vygeneruje náhodný graf o tolika uzlech," +
                " kolik je uvedeno v poli Počet uzlů,\n" +
                "  Smazat vše – vyčistí kreslicí plátno, Vyřešit - prohledá a vyhodnotí nakreslený graf, Příklady – sada připravených příkladů\n\n" +

                "  The application is used to search for Hamiltonian graphs using backtracking. A Hamiltonian graph exists" +
                " if it is possible to go through the graph \n" +
                "  in a loop and visit each node exactly once and return to the starting node. Graphs can be drawn or" +
                " generated within the application. \n" +
                "  It also contains a set of prepared examples.\n" +
                "\n  Controls:\n" +
                "  Left mouse button  is used to select either a singular node or multiple nodes by dragging the mouse around them.\n" +
                "  Right mouse button opens vertical menu with buttons: New, Delete, Connect.\n" +
                " \n  Buttons functions:\n" +
                "  CZ/EN – switches languages: English or Czech, New – creates new node, Delete – deletes selected nodes,\n" +
                "  Connect – connects selected nodes with edges, Save – saves the drawn graph so that it can be searched,\n" +
                "  Generate – generates a random graph with as many nodes as specified in the Vertex count field,\n" +
                "  Clear – removes all from drawing screen, Solve – searches and solves drawn graph," +
                "\n  Example - opens menu with prepared examples");
    }


}